@@ -1,14 +1,14 @@
 import os
 
-from distutils.core import Extension
-from distutils.core import setup
+from setuptools import Extension
+from setuptools import setup
 
 
 # =====
 if __name__ == "__main__":
     setup(
         name="ustreamer",
-        version="4.9",
+        version="5.4",
         description="uStreamer tools",
         author="Maxim Devaev",
         author_email="mdevaev@gmail.com",
