@@ -1,8 +1,8 @@
 /*****************************************************************************
 #                                                                            #
-#    uStreamer - Lightweight and fast MJPG-HTTP streamer.                    #
+#    uStreamer - Lightweight and fast MJPEG-HTTP streamer.                   #
 #                                                                            #
-#    Copyright (C) 2018-2021  Maxim Devaev <mdevaev@gmail.com>               #
+#    Copyright (C) 2018-2022  Maxim Devaev <mdevaev@gmail.com>               #
 #                                                                            #
 #    This program is free software: you can redistribute it and/or modify    #
 #    it under the terms of the GNU General Public License as published by    #
@@ -27,12 +27,14 @@ static const struct {
 	const char *name;
 	const encoder_type_e type;
 } _ENCODER_TYPES[] = {
-	{"CPU",		ENCODER_TYPE_CPU},
-	{"HW",		ENCODER_TYPE_HW},
-#	ifdef WITH_OMX
-	{"OMX",		ENCODER_TYPE_OMX},
-#	endif
-	{"NOOP",	ENCODER_TYPE_NOOP},
+	{"CPU",			ENCODER_TYPE_CPU},
+	{"HW",			ENCODER_TYPE_HW},
+	{"M2M-VIDEO",	ENCODER_TYPE_M2M_VIDEO},
+	{"M2M-IMAGE",	ENCODER_TYPE_M2M_IMAGE},
+	{"M2M-MJPEG",	ENCODER_TYPE_M2M_VIDEO},
+	{"M2M-JPEG",	ENCODER_TYPE_M2M_IMAGE},
+	{"OMX",			ENCODER_TYPE_M2M_IMAGE},
+	{"NOOP",		ENCODER_TYPE_NOOP},
 };
 
 
@@ -60,16 +62,14 @@ encoder_s *encoder_init(void) {
 }
 
 void encoder_destroy(encoder_s *enc) {
-#	ifdef WITH_OMX
-	if (ER(omxs)) {
-		for (unsigned index = 0; index < ER(n_omxs); ++index) {
-			if (ER(omxs[index])) {
-				omx_encoder_destroy(ER(omxs[index]));
+	if (ER(m2ms)) {
+		for (unsigned index = 0; index < ER(n_m2ms); ++index) {
+			if (ER(m2ms[index])) {
+				m2m_encoder_destroy(ER(m2ms[index]));
 			}
 		}
-		free(ER(omxs));
+		free(ER(m2ms));
 	}
-#	endif
 	A_MUTEX_DESTROY(&ER(mutex));
 	free(enc->run);
 	free(enc);
@@ -113,63 +113,30 @@ workers_pool_s *encoder_workers_pool_init(encoder_s *enc, device_s *dev) {
 		}
 		quality = DR(jpeg_quality);
 		n_workers = 1;
-	}
-#	ifdef WITH_OMX
-	else if (type == ENCODER_TYPE_OMX) {
-		if (align_size(DR(width), 32) != DR(width)) {
-			LOG_INFO("Switching to CPU encoder: OMX can't handle width=%u ...", DR(width));
-			goto use_cpu;
+
+	} else if (type == ENCODER_TYPE_M2M_VIDEO || type == ENCODER_TYPE_M2M_IMAGE) {
+		LOG_DEBUG("Preparing M2M-%s encoder ...", (type == ENCODER_TYPE_M2M_VIDEO ? "VIDEO" : "IMAGE"));
+		if (ER(m2ms) == NULL) {
+			A_CALLOC(ER(m2ms), n_workers);
 		}
-
-		LOG_DEBUG("Preparing OMX encoder ...");
-
-		if (n_workers > OMX_MAX_ENCODERS) {
-			LOG_INFO("OMX encoder sets limit for worker threads: %u", OMX_MAX_ENCODERS);
-			n_workers = OMX_MAX_ENCODERS;
-		}
-
-		if (ER(omxs) == NULL) {
-			A_CALLOC(ER(omxs), OMX_MAX_ENCODERS);
-		}
-
-		// Начинаем с нуля и доинициализируем на следующих заходах при необходимости
-		for (; ER(n_omxs) < n_workers; ++ER(n_omxs)) {
-			if ((ER(omxs[ER(n_omxs)]) = omx_encoder_init()) == NULL) {
-				LOG_ERROR("Can't initialize OMX encoder, falling back to CPU");
-				goto force_cpu;
+		for (; ER(n_m2ms) < n_workers; ++ER(n_m2ms)) {
+			// Начинаем с нуля и доинициализируем на следующих заходах при необходимости
+			char name[32];
+			snprintf(name, 32, "JPEG-%u", ER(n_m2ms));
+			if (type == ENCODER_TYPE_M2M_VIDEO) {
+				ER(m2ms[ER(n_m2ms)]) = m2m_mjpeg_encoder_init(name, enc->m2m_path, quality);
+			} else {
+				ER(m2ms[ER(n_m2ms)]) = m2m_jpeg_encoder_init(name, enc->m2m_path, quality);
 			}
 		}
 
-		frame_s frame;
-		MEMSET_ZERO(frame);
-		frame.width = DR(width);
-		frame.height = DR(height);
-		frame.format = DR(format);
-		frame.stride = DR(stride);
-
-		for (unsigned index = 0; index < ER(n_omxs); ++index) {
-			int omx_error = omx_encoder_prepare(ER(omxs[index]), &frame, quality);
-			if (omx_error == -2) {
-				goto use_cpu;
-			} else if (omx_error < 0) {
-				goto force_cpu;
-			}
-		}
-	}
-#	endif
-	else if (type == ENCODER_TYPE_NOOP) {
+	} else if (type == ENCODER_TYPE_NOOP) {
 		n_workers = 1;
 		quality = 0;
 	}
 
 	goto ok;
 
-#	ifdef WITH_OMX
-	force_cpu:
-		LOG_ERROR("Forced CPU encoder permanently");
-		cpu_forced = true;
-#	endif
-
 	use_cpu:
 		type = ENCODER_TYPE_CPU;
 		quality = dev->jpeg_quality;
@@ -235,55 +202,47 @@ static bool _worker_run_job(worker_s *wr) {
 
 #	define ER(_next) job->enc->run->_next
 
-	LOG_DEBUG("Worker %s compressing JPEG from buffer %u ...", wr->name, job->hw->buf_info.index);
-
 	assert(ER(type) != ENCODER_TYPE_UNKNOWN);
-	assert(src->used > 0);
-
-	frame_copy_meta(src, dest);
-	dest->format = V4L2_PIX_FMT_JPEG;
-	dest->stride = 0;
-	dest->encode_begin_ts = get_now_monotonic();
-	dest->used = 0;
 
 	if (ER(type) == ENCODER_TYPE_CPU) {
-		LOG_VERBOSE("Compressing buffer using CPU");
+		LOG_VERBOSE("Compressing JPEG using CPU: worker=%s, buffer=%u",
+			wr->name, job->hw->buf.index);
 		cpu_encoder_compress(src, dest, ER(quality));
+
 	} else if (ER(type) == ENCODER_TYPE_HW) {
-		LOG_VERBOSE("Compressing buffer using HW (just copying)");
+		LOG_VERBOSE("Compressing JPEG using HW (just copying): worker=%s, buffer=%u",
+			wr->name, job->hw->buf.index);
 		hw_encoder_compress(src, dest);
-	}
-#	ifdef WITH_OMX
-	else if (ER(type) == ENCODER_TYPE_OMX) {
-		LOG_VERBOSE("Compressing buffer using OMX");
-		if (omx_encoder_compress(ER(omxs[wr->number]), src, dest) < 0) {
+
+	} else if (ER(type) == ENCODER_TYPE_M2M_VIDEO || ER(type) == ENCODER_TYPE_M2M_IMAGE) {
+		LOG_VERBOSE("Compressing JPEG using M2M-%s: worker=%s, buffer=%u",
+			(ER(type) == ENCODER_TYPE_M2M_VIDEO ? "VIDEO" : "IMAGE"), wr->name, job->hw->buf.index);
+		if (m2m_encoder_compress(ER(m2ms[wr->number]), src, dest, false) < 0) {
 			goto error;
 		}
-	}
-#	endif
-	else if (ER(type) == ENCODER_TYPE_NOOP) {
-		LOG_VERBOSE("Compressing buffer using NOOP (do nothing)");
+
+	} else if (ER(type) == ENCODER_TYPE_NOOP) {
+		LOG_VERBOSE("Compressing JPEG using NOOP (do nothing): worker=%s, buffer=%u",
+			wr->name, job->hw->buf.index);
 		usleep(5000); // Просто чтобы работала логика desired_fps
+		dest->encode_end_ts = get_now_monotonic();
 	}
 
-	dest->encode_end_ts = get_now_monotonic();
 	LOG_VERBOSE("Compressed new JPEG: size=%zu, time=%0.3Lf, worker=%s, buffer=%u",
 		job->dest->used,
 		job->dest->encode_end_ts - job->dest->encode_begin_ts,
 		wr->name,
-		job->hw->buf_info.index);
+		job->hw->buf.index);
 
 	return true;
 
-#	ifdef WITH_OMX
 	error:
-		LOG_ERROR("Compression failed: worker=%s, buffer=%u", wr->name, job->hw->buf_info.index);
+		LOG_ERROR("Compression failed: worker=%s, buffer=%u", wr->name, job->hw->buf.index);
 		LOG_ERROR("Error while compressing buffer, falling back to CPU");
 		A_MUTEX_LOCK(&ER(mutex));
 		ER(cpu_forced) = true;
 		A_MUTEX_UNLOCK(&ER(mutex));
 		return false;
-#	endif
 
 #	undef ER
 }
