@@ -1,8 +1,8 @@
 /*****************************************************************************
 #                                                                            #
-#    uStreamer - Lightweight and fast MJPG-HTTP streamer.                    #
+#    uStreamer - Lightweight and fast MJPEG-HTTP streamer.                   #
 #                                                                            #
-#    Copyright (C) 2018-2021  Maxim Devaev <mdevaev@gmail.com>               #
+#    Copyright (C) 2018-2022  Maxim Devaev <mdevaev@gmail.com>               #
 #                                                                            #
 #    This program is free software: you can redistribute it and/or modify    #
 #    it under the terms of the GNU General Public License as published by    #
@@ -36,7 +36,8 @@ static void _http_callback_stream(struct evhttp_request *request, void *v_server
 static void _http_callback_stream_write(struct bufferevent *buf_event, void *v_ctx);
 static void _http_callback_stream_error(struct bufferevent *buf_event, short what, void *v_ctx);
 
-static void _http_exposed_refresh(int fd, short event, void *v_server);
+static void _http_request_watcher(int fd, short event, void *v_server);
+static void _http_refresher(int fd, short event, void *v_server);
 static void _http_queue_send_stream(server_s *server, bool stream_updated, bool frame_updated);
 
 static bool _expose_new_frame(server_s *server);
@@ -75,19 +76,24 @@ server_s *server_init(stream_s *stream) {
 	assert(!evthread_use_pthreads());
 	assert((run->base = event_base_new()));
 	assert((run->http = evhttp_new(run->base)));
-	evhttp_set_allowed_methods(run->http, EVHTTP_REQ_GET|EVHTTP_REQ_HEAD);
+	evhttp_set_allowed_methods(run->http, EVHTTP_REQ_GET|EVHTTP_REQ_HEAD|EVHTTP_REQ_OPTIONS);
 	return server;
 }
 
 void server_destroy(server_s *server) {
-	if (RUN(refresh)) {
-		event_del(RUN(refresh));
-		event_free(RUN(refresh));
+	if (RUN(refresher)) {
+		event_del(RUN(refresher));
+		event_free(RUN(refresher));
+	}
+
+	if (RUN(request_watcher)) {
+		event_del(RUN(request_watcher));
+		event_free(RUN(request_watcher));
 	}
 
 	evhttp_free(RUN(http));
-	if (RUN(unix_fd)) {
-		close(RUN(unix_fd));
+	if (RUN(ext_fd)) {
+		close(RUN(ext_fd));
 	}
 	event_base_free(RUN(base));
 
@@ -128,18 +134,23 @@ int server_listen(server_s *server) {
 	EX(notify_last_width) = EX(frame->width);
 	EX(notify_last_height) = EX(frame->height);
 
+	if (server->exit_on_no_clients > 0) {
+		RUN(last_request_ts) = get_now_monotonic();
+		struct timeval interval = {0};
+		interval.tv_usec = 100000;
+		assert((RUN(request_watcher) = event_new(RUN(base), -1, EV_PERSIST, _http_request_watcher, server)));
+		assert(!event_add(RUN(request_watcher), &interval));
+	}
+
 	{
-		struct timeval refresh_interval;
-
-		refresh_interval.tv_sec = 0;
+		struct timeval interval = {0};
 		if (STREAM(dev->desired_fps) > 0) {
-			refresh_interval.tv_usec = 1000000 / (STREAM(dev->desired_fps) * 2);
+			interval.tv_usec = 1000000 / (STREAM(dev->desired_fps) * 2);
 		} else {
-			refresh_interval.tv_usec = 16000; // ~60fps
+			interval.tv_usec = 16000; // ~60fps
 		}
-
-		assert((RUN(refresh) = event_new(RUN(base), -1, EV_PERSIST, _http_exposed_refresh, server)));
-		assert(!event_add(RUN(refresh), &refresh_interval));
+		assert((RUN(refresher) = event_new(RUN(base), -1, EV_PERSIST, _http_refresher, server)));
+		assert(!event_add(RUN(refresher), &interval));
 	}
 
 	evhttp_set_timeout(RUN(http), server->timeout);
@@ -160,7 +171,7 @@ int server_listen(server_s *server) {
 
 	if (server->unix_path[0] != '\0') {
 		LOG_DEBUG("Binding HTTP to UNIX socket '%s' ...", server->unix_path);
-		if ((RUN(unix_fd) = evhttp_my_bind_unix(
+		if ((RUN(ext_fd) = evhttp_my_bind_unix(
 			RUN(http),
 			server->unix_path,
 			server->unix_rm,
@@ -169,9 +180,16 @@ int server_listen(server_s *server) {
 			return -1;
 		}
 		LOG_INFO("Listening HTTP on UNIX socket '%s'", server->unix_path);
-		if (server->tcp_nodelay) {
-			LOG_ERROR("TCP_NODELAY flag can't be used with UNIX socket and will be ignored");
+
+#	ifdef WITH_SYSTEMD
+	} else if (server->systemd) {
+		LOG_DEBUG("Binding HTTP to systemd socket ...");
+		if ((RUN(ext_fd) = evhttp_my_bind_systemd(RUN(http))) < 0) {
+			return -1;
 		}
+		LOG_INFO("Listening systemd socket ...");
+#	endif
+
 	} else {
 		LOG_DEBUG("Binding HTTP to [%s]:%u ...", server->host, server->port);
 		if (evhttp_bind_socket(RUN(http), server->host, server->port) < 0) {
@@ -194,12 +212,36 @@ void server_loop_break(server_s *server) {
 	event_base_loopbreak(RUN(base));
 }
 
+#define GET_HEADER(_key) \
+	evhttp_find_header(evhttp_request_get_input_headers(request), _key)
+
 #define ADD_HEADER(_key, _value) \
 	assert(!evhttp_add_header(evhttp_request_get_output_headers(request), _key, _value))
 
 static int _http_preprocess_request(struct evhttp_request *request, server_s *server) {
+	RUN(last_request_ts) = get_now_monotonic();
+
+	if (server->allow_origin[0] != '\0') {
+		const char *cors_headers = GET_HEADER("Access-Control-Request-Headers");
+		const char *cors_method = GET_HEADER("Access-Control-Request-Method");
+
+		ADD_HEADER("Access-Control-Allow-Origin", server->allow_origin);
+		ADD_HEADER("Access-Control-Allow-Credentials", "true");
+		if (cors_headers != NULL) {
+			ADD_HEADER("Access-Control-Allow-Headers", cors_headers);
+		}
+		if (cors_method != NULL) {
+			ADD_HEADER("Access-Control-Allow-Methods", cors_method);
+		}
+	}
+
+	if (evhttp_request_get_command(request) == EVHTTP_REQ_OPTIONS) {
+		evhttp_send_reply(request, HTTP_OK, "OK", NULL);
+		return -1;
+	}
+
 	if (RUN(auth_token)) {
-		const char *token = evhttp_find_header(evhttp_request_get_input_headers(request), "Authorization");
+		const char *token = GET_HEADER("Authorization");
 
 		if (token == NULL || strcmp(token, RUN(auth_token)) != 0) {
 			ADD_HEADER("WWW-Authenticate", "Basic realm=\"Restricted area\"");
@@ -208,8 +250,8 @@ static int _http_preprocess_request(struct evhttp_request *request, server_s *se
 		}
 	}
 
-	if (evhttp_request_get_command(request) == EVHTTP_REQ_HEAD) { \
-		evhttp_send_reply(request, HTTP_OK, "OK", NULL); \
+	if (evhttp_request_get_command(request) == EVHTTP_REQ_HEAD) {
+		evhttp_send_reply(request, HTTP_OK, "OK", NULL);
 		return -1;
 	}
 
@@ -317,6 +359,10 @@ static void _http_callback_static(struct evhttp_request *request, void *v_server
 			goto not_found;
 		}
 
+		// evbuffer_add_file() owns the resulting file descriptor
+		// and will close it when finished transferring data
+		fd = -1;
+
 		ADD_HEADER("Content-Type", guess_mime_type(static_path));
 		evhttp_send_reply(request, HTTP_OK, "OK", buf);
 		goto cleanup;
@@ -369,7 +415,6 @@ static void _http_callback_state(struct evhttp_request *request, void *v_server)
 		enc_quality
 	));
 
-#	ifdef WITH_OMX
 	if (STREAM(run->h264)) {
 		assert(evbuffer_add_printf(buf,
 			" \"h264\": {\"bitrate\": %u, \"gop\": %u, \"online\": %s},",
@@ -378,14 +423,8 @@ static void _http_callback_state(struct evhttp_request *request, void *v_server)
 			bool_to_string(atomic_load(&STREAM(run->h264->online)))
 		));
 	}
-#	endif
 
-	if (
-		STREAM(sink)
-#	ifdef WITH_OMX
-		|| STREAM(h264_sink)
-#	endif
-	) {
+	if (STREAM(sink) || STREAM(h264_sink)) {
 		assert(evbuffer_add_printf(buf, " \"sinks\": {"));
 		if (STREAM(sink)) {
 			assert(evbuffer_add_printf(buf,
@@ -393,7 +432,6 @@ static void _http_callback_state(struct evhttp_request *request, void *v_server)
 				bool_to_string(atomic_load(&STREAM(sink->has_clients)))
 			));
 		}
-#	ifdef WITH_OMX
 		if (STREAM(h264_sink)) {
 			assert(evbuffer_add_printf(buf,
 				"%s\"h264\": {\"has_clients\": %s}",
@@ -401,7 +439,6 @@ static void _http_callback_state(struct evhttp_request *request, void *v_server)
 				bool_to_string(atomic_load(&STREAM(h264_sink->has_clients)))
 			));
 		}
-#	endif
 		assert(evbuffer_add_printf(buf, "},"));
 	}
 
@@ -421,13 +458,14 @@ static void _http_callback_state(struct evhttp_request *request, void *v_server)
 	LIST_ITERATE(RUN(stream_clients), client, {
 		assert(evbuffer_add_printf(buf,
 			"\"%" PRIx64 "\": {\"fps\": %u, \"extra_headers\": %s, \"advance_headers\": %s,"
-			" \"dual_final_frames\": %s, \"zero_data\": %s}%s",
+			" \"dual_final_frames\": %s, \"zero_data\": %s, \"key\": \"%s\"}%s",
 			client->id,
 			client->fps,
 			bool_to_string(client->extra_headers),
 			bool_to_string(client->advance_headers),
 			bool_to_string(client->dual_final_frames),
 			bool_to_string(client->zero_data),
+			(client->key != NULL ? client->key : "0"),
 			(client->next ? ", " : "")
 		));
 	});
@@ -448,9 +486,6 @@ static void _http_callback_snapshot(struct evhttp_request *request, void *v_serv
 	assert((buf = evbuffer_new()));
 	assert(!evbuffer_add(buf, (const void *)EX(frame->data), EX(frame->used)));
 
-	if (server->allow_origin[0] != '\0') {
-		ADD_HEADER("Access-Control-Allow-Origin", server->allow_origin);
-	}
 	ADD_HEADER("Cache-Control", "no-store, no-cache, must-revalidate, proxy-revalidate, pre-check=0, post-check=0, max-age=0");
 	ADD_HEADER("Pragma", "no-cache");
 	ADD_HEADER("Expires", "Mon, 3 Jan 2000 12:34:56 GMT");
@@ -541,7 +576,7 @@ static void _http_callback_stream(struct evhttp_request *request, void *v_server
 			client->hostport, client->id, RUN(stream_clients_count));
 
 		struct bufferevent *buf_event = evhttp_connection_get_bufferevent(conn);
-		if (server->tcp_nodelay && !RUN(unix_fd)) {
+		if (server->tcp_nodelay && !RUN(ext_fd)) {
 			evutil_socket_t fd;
 			int on = 1;
 
@@ -564,6 +599,7 @@ static void _http_callback_stream_write(struct bufferevent *buf_event, void *v_c
 #	define BOUNDARY "boundarydonotcross"
 
 	stream_client_s *client = (stream_client_s *)v_client;
+	struct evhttp_request *request = client->request; // for GET_HEADER
 	server_s *server = client->server;
 
 	long double now = get_now_monotonic();
@@ -591,7 +627,7 @@ static void _http_callback_stream_write(struct bufferevent *buf_event, void *v_c
 	// будущего фрейма сразу после данных текущего, чтобы триггернуть отрисовку.
 	// Естественным следствием этого является невозможность установки заголовка
 	// Content-Length, так как предсказывать будущее мы еще не научились.
-	// Его наличие не требуется RFC, однако никаких стандартов на MJPG over HTTP
+	// Его наличие не требуется RFC, однако никаких стандартов на MJPEG over HTTP
 	// в природе не существует, и никто не может гарантировать, что отсутствие
 	// Content-Length не сломает вещание для каких-нибудь маргинальных браузеров.
 	//
@@ -604,9 +640,24 @@ static void _http_callback_stream_write(struct bufferevent *buf_event, void *v_c
 
 	if (client->need_initial) {
 		assert(evbuffer_add_printf(buf, "HTTP/1.0 200 OK" RN));
+		
 		if (client->server->allow_origin[0] != '\0') {
-			assert(evbuffer_add_printf(buf, "Access-Control-Allow-Origin: %s" RN, client->server->allow_origin));
+			const char *cors_headers = GET_HEADER("Access-Control-Request-Headers");
+			const char *cors_method = GET_HEADER("Access-Control-Request-Method");
+
+			assert(evbuffer_add_printf(buf,
+				"Access-Control-Allow-Origin: %s" RN
+				"Access-Control-Allow-Credentials: true" RN,
+				client->server->allow_origin				
+			));
+			if (cors_headers != NULL) {
+				assert(evbuffer_add_printf(buf, "Access-Control-Allow-Headers: %s" RN, cors_headers));
+			}
+			if (cors_method != NULL) {
+				assert(evbuffer_add_printf(buf, "Access-Control-Allow-Methods: %s" RN, cors_method));
+			}
 		}
+
 		assert(evbuffer_add_printf(buf,
 			"Cache-Control: no-store, no-cache, must-revalidate, proxy-revalidate, pre-check=0, post-check=0, max-age=0" RN
 			"Pragma: no-cache" RN
@@ -769,7 +820,21 @@ static void _http_queue_send_stream(server_s *server, bool stream_updated, bool
 	}
 }
 
-static void _http_exposed_refresh(UNUSED int fd, UNUSED short what, void *v_server) {
+static void _http_request_watcher(UNUSED int fd, UNUSED short what, void *v_server) {
+	server_s *server = (server_s *)v_server;
+	const long double now = get_now_monotonic();
+
+	if (stream_has_clients(RUN(stream))) {
+		RUN(last_request_ts) = now;
+	} else if (RUN(last_request_ts) + server->exit_on_no_clients < now) {
+		LOG_INFO("HTTP: No requests or HTTP/sink clients found in last %u seconds, exiting ...",
+			server->exit_on_no_clients);
+		process_suicide();
+		RUN(last_request_ts) = now;
+	}
+}
+
+static void _http_refresher(UNUSED int fd, UNUSED short what, void *v_server) {
 	server_s *server = (server_s *)v_server;
 	bool stream_updated = false;
 	bool frame_updated = false;
@@ -865,7 +930,7 @@ static char *_http_get_client_hostport(struct evhttp_request *request) {
 		assert(addr = strdup(peer));
 	}
 
-	const char *xff = evhttp_find_header(evhttp_request_get_input_headers(request), "X-Forwarded-For");
+	const char *xff = GET_HEADER("X-Forwarded-For");
 	if (xff) {
 		if (addr) {
 			free(addr);
@@ -888,3 +953,5 @@ static char *_http_get_client_hostport(struct evhttp_request *request) {
 	free(addr);
 	return hostport;
 }
+
+#undef GET_HEADER
