@@ -1,8 +1,8 @@
 /*****************************************************************************
 #                                                                            #
-#    uStreamer - Lightweight and fast MJPG-HTTP streamer.                    #
+#    uStreamer - Lightweight and fast MJPEG-HTTP streamer.                   #
 #                                                                            #
-#    Copyright (C) 2018-2021  Maxim Devaev <mdevaev@gmail.com>               #
+#    Copyright (C) 2018-2022  Maxim Devaev <mdevaev@gmail.com>               #
 #                                                                            #
 #    This program is free software: you can redistribute it and/or modify    #
 #    it under the terms of the GNU General Public License as published by    #
@@ -71,6 +71,9 @@
 #include "uri.h"
 #include "mime.h"
 #include "static.h"
+#ifdef WITH_SYSTEMD
+#	include "systemd/systemd.h"
+#endif
 
 
 typedef struct stream_client_sx {
@@ -112,11 +115,17 @@ typedef struct {
 typedef struct {
 	struct event_base	*base;
 	struct evhttp		*http;
-	evutil_socket_t		unix_fd;
+	evutil_socket_t		ext_fd; // Unix or socket activation
+
 	char				*auth_token;
-	struct event		*refresh;
+
+	struct event		*request_watcher;
+	long double			last_request_ts;
+
+	struct event		*refresher;
 	stream_s			*stream;
 	exposed_s			*exposed;
+
 	stream_client_s		*stream_clients;
 	unsigned			stream_clients_count;
 } server_runtime_s;
@@ -124,9 +133,15 @@ typedef struct {
 typedef struct server_sx {
 	char		*host;
 	unsigned	port;
+
 	char		*unix_path;
 	bool		unix_rm;
 	mode_t		unix_mode;
+
+#	ifdef WITH_SYSTEMD
+	bool		systemd;
+#	endif
+
 	bool		tcp_nodelay;
 	unsigned	timeout;
 
@@ -140,6 +155,7 @@ typedef struct server_sx {
 	unsigned	fake_height;
 
 	bool		notify_parent;
+	unsigned	exit_on_no_clients;
 
 	server_runtime_s *run;
 } server_s;
