@@ -1,8 +1,8 @@
 /*****************************************************************************
 #                                                                            #
-#    uStreamer - Lightweight and fast MJPG-HTTP streamer.                    #
+#    uStreamer - Lightweight and fast MJPEG-HTTP streamer.                   #
 #                                                                            #
-#    Copyright (C) 2018-2021  Maxim Devaev <mdevaev@gmail.com>               #
+#    Copyright (C) 2018-2022  Maxim Devaev <mdevaev@gmail.com>               #
 #                                                                            #
 #    This program is free software: you can redistribute it and/or modify    #
 #    it under the terms of the GNU General Public License as published by    #
@@ -38,9 +38,7 @@ enum _OPT_VALUES {
 	_O_WORKERS = 'w',
 	_O_QUALITY = 'q',
 	_O_ENCODER = 'c',
-#	ifdef WITH_OMX
-	_O_GLITCHED_RESOLUTIONS = 'g',
-#	endif
+	_O_GLITCHED_RESOLUTIONS = 'g', // Deprecated
 	_O_BLANK = 'k',
 	_O_LAST_AS_BLANK = 'K',
 	_O_SLOWDOWN = 'l',
@@ -50,6 +48,9 @@ enum _OPT_VALUES {
 	_O_UNIX = 'U',
 	_O_UNIX_RM = 'D',
 	_O_UNIX_MODE = 'M',
+#	ifdef WITH_SYSTEMD
+	_O_SYSTEMD = 'S',
+#	endif
 	_O_DROP_SAME_FRAMES = 'e',
 	_O_FAKE_RESOLUTION = 'R',
 
@@ -60,6 +61,7 @@ enum _OPT_VALUES {
 
 	_O_DEVICE_TIMEOUT = 10000,
 	_O_DEVICE_ERROR_DELAY,
+	_O_M2M_DEVICE,
 
 	_O_IMAGE_DEFAULT,
 	_O_BRIGHTNESS,
@@ -91,11 +93,10 @@ enum _OPT_VALUES {
 		_O_##_prefix##_TIMEOUT,
 	ADD_SINK(SINK)
 	ADD_SINK(RAW_SINK)
-#	ifdef WITH_OMX
 	ADD_SINK(H264_SINK)
 	_O_H264_BITRATE,
 	_O_H264_GOP,
-#	endif
+	_O_H264_M2M_DEVICE,
 #	undef ADD_SINK
 
 #	ifdef WITH_GPIO
@@ -109,6 +110,7 @@ enum _OPT_VALUES {
 #	ifdef HAS_PDEATHSIG
 	_O_EXIT_ON_PARENT_DEATH,
 #	endif
+	_O_EXIT_ON_NO_CLIENTS,
 #	ifdef WITH_SETPROCTITLE
 	_O_PROCESS_NAME_PREFIX,
 #	endif
@@ -139,14 +141,13 @@ static const struct option _LONG_OPTS[] = {
 	{"workers",					required_argument,	NULL,	_O_WORKERS},
 	{"quality",					required_argument,	NULL,	_O_QUALITY},
 	{"encoder",					required_argument,	NULL,	_O_ENCODER},
-#	ifdef WITH_OMX
-	{"glitched-resolutions",	required_argument,	NULL,	_O_GLITCHED_RESOLUTIONS},
-#	endif
+	{"glitched-resolutions",	required_argument,	NULL,	_O_GLITCHED_RESOLUTIONS}, // Deprecated
 	{"blank",					required_argument,	NULL,	_O_BLANK},
 	{"last-as-blank",			required_argument,	NULL,	_O_LAST_AS_BLANK},
 	{"slowdown",				no_argument,		NULL,	_O_SLOWDOWN},
 	{"device-timeout",			required_argument,	NULL,	_O_DEVICE_TIMEOUT},
 	{"device-error-delay",		required_argument,	NULL,	_O_DEVICE_ERROR_DELAY},
+	{"m2m-device",				required_argument,	NULL,	_O_M2M_DEVICE},
 
 	{"image-default",			no_argument,		NULL,	_O_IMAGE_DEFAULT},
 	{"brightness",				required_argument,	NULL,	_O_BRIGHTNESS},
@@ -159,7 +160,7 @@ static const struct option _LONG_OPTS[] = {
 	{"white-balance",			required_argument,	NULL,	_O_WHITE_BALANCE},
 	{"gain",					required_argument,	NULL,	_O_GAIN},
 	{"color-effect",			required_argument,	NULL,	_O_COLOR_EFFECT},
-	{"rotate",				required_argument,	NULL,	_O_ROTATE},
+	{"rotate",					required_argument,	NULL,	_O_ROTATE},
 	{"flip-vertical",			required_argument,	NULL,	_O_FLIP_VERTICAL},
 	{"flip-horizontal",			required_argument,	NULL,	_O_FLIP_HORIZONTAL},
 
@@ -168,6 +169,9 @@ static const struct option _LONG_OPTS[] = {
 	{"unix",					required_argument,	NULL,	_O_UNIX},
 	{"unix-rm",					no_argument,		NULL,	_O_UNIX_RM},
 	{"unix-mode",				required_argument,	NULL,	_O_UNIX_MODE},
+#	ifdef WITH_SYSTEMD
+	{"systemd",					no_argument,		NULL,	_O_SYSTEMD},
+#	endif
 	{"user",					required_argument,	NULL,	_O_USER},
 	{"passwd",					required_argument,	NULL,	_O_PASSWD},
 	{"static",					required_argument,	NULL,	_O_STATIC},
@@ -185,11 +189,10 @@ static const struct option _LONG_OPTS[] = {
 		{_opt "sink-timeout",		required_argument,	NULL,	_O_##_prefix##_TIMEOUT},
 	ADD_SINK("", SINK)
 	ADD_SINK("raw-", RAW_SINK)
-#	ifdef WITH_OMX
 	ADD_SINK("h264-", H264_SINK)
 	{"h264-bitrate",			required_argument,	NULL,	_O_H264_BITRATE},
 	{"h264-gop",				required_argument,	NULL,	_O_H264_GOP},
-#	endif
+	{"h264-m2m-device",			required_argument,	NULL,	_O_H264_M2M_DEVICE},
 #	undef ADD_SINK
 
 #	ifdef WITH_GPIO
@@ -203,6 +206,7 @@ static const struct option _LONG_OPTS[] = {
 #	ifdef HAS_PDEATHSIG
 	{"exit-on-parent-death",	no_argument,		NULL,	_O_EXIT_ON_PARENT_DEATH},
 #	endif
+	{"exit-on-no-clients",		required_argument,	NULL,	_O_EXIT_ON_NO_CLIENTS},
 #	ifdef WITH_SETPROCTITLE
 	{"process-name-prefix",		required_argument,	NULL,	_O_PROCESS_NAME_PREFIX},
 #	endif
@@ -250,9 +254,7 @@ void options_destroy(options_s *options) {
 		}
 	ADD_SINK(sink);
 	ADD_SINK(raw_sink);
-#	ifdef WITH_OMX
 	ADD_SINK(h264_sink);
-#	endif
 #	undef ADD_SINK
 
 	if (options->blank) {
@@ -345,9 +347,7 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 		unsigned _prefix##_timeout = 1;
 	ADD_SINK(sink);
 	ADD_SINK(raw_sink);
-#	ifdef WITH_OMX
 	ADD_SINK(h264_sink);
-#	endif
 #	undef ADD_SINK
 
 #	ifdef WITH_SETPROCTITLE
@@ -376,14 +376,13 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 			case _O_WORKERS:		OPT_NUMBER("--workers", enc->n_workers, 1, 32, 0);
 			case _O_QUALITY:		OPT_NUMBER("--quality", dev->jpeg_quality, 1, 100, 0);
 			case _O_ENCODER:		OPT_PARSE("encoder type", enc->type, encoder_parse_type, ENCODER_TYPE_UNKNOWN, ENCODER_TYPES_STR);
-#			ifdef WITH_OMX
-			case _O_GLITCHED_RESOLUTIONS: break;
-#			endif
+			case _O_GLITCHED_RESOLUTIONS: break; // Deprecated
 			case _O_BLANK:				OPT_SET(blank_path, optarg);
 			case _O_LAST_AS_BLANK:		OPT_NUMBER("--last-as-blank", stream->last_as_blank, 0, 86400, 0);
 			case _O_SLOWDOWN:			OPT_SET(stream->slowdown, true);
 			case _O_DEVICE_TIMEOUT:		OPT_NUMBER("--device-timeout", dev->timeout, 1, 60, 0);
 			case _O_DEVICE_ERROR_DELAY:	OPT_NUMBER("--device-error-delay", stream->error_delay, 1, 60, 0);
+			case _O_M2M_DEVICE:			OPT_SET(enc->m2m_path, optarg);
 
 			case _O_IMAGE_DEFAULT:
 				OPT_CTL_DEFAULT_NOBREAK(brightness);
@@ -419,6 +418,9 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 			case _O_UNIX:				OPT_SET(server->unix_path, optarg);
 			case _O_UNIX_RM:			OPT_SET(server->unix_rm, true);
 			case _O_UNIX_MODE:			OPT_NUMBER("--unix-mode", server->unix_mode, INT_MIN, INT_MAX, 8);
+#			ifdef WITH_SYSTEMD
+			case _O_SYSTEMD:			OPT_SET(server->systemd, true);
+#			endif
 			case _O_USER:				OPT_SET(server->user, optarg);
 			case _O_PASSWD:				OPT_SET(server->passwd, optarg);
 			case _O_STATIC:				OPT_SET(server->static_path, optarg);
@@ -436,11 +438,10 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 				case _O_##_up##_TIMEOUT:	OPT_NUMBER("--" #_opt "sink-timeout", _lp##_timeout, 1, 60, 0);
 			ADD_SINK("", sink, SINK)
 			ADD_SINK("raw-", raw_sink, RAW_SINK)
-#			ifdef WITH_OMX
 			ADD_SINK("h264-", h264_sink, H264_SINK)
-			case _O_H264_BITRATE:	OPT_NUMBER("--h264-bitrate", stream->h264_bitrate, 100, 16000, 0);
-			case _O_H264_GOP:		OPT_NUMBER("--h264-gop", stream->h264_gop, 0, 60, 0);
-#			endif
+			case _O_H264_BITRATE:		OPT_NUMBER("--h264-bitrate", stream->h264_bitrate, 25, 20000, 0);
+			case _O_H264_GOP:			OPT_NUMBER("--h264-gop", stream->h264_gop, 0, 60, 0);
+			case _O_H264_M2M_DEVICE:	OPT_SET(stream->h264_m2m_path, optarg);
 #			undef ADD_SINK
 
 #			ifdef WITH_GPIO
@@ -458,6 +459,7 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 				};
 				break;
 #			endif
+			case _O_EXIT_ON_NO_CLIENTS:		OPT_NUMBER("--exit-on-no-clients", server->exit_on_no_clients, 0, 86400, 0);
 #			ifdef WITH_SETPROCTITLE
 			case _O_PROCESS_NAME_PREFIX:	OPT_SET(process_name_prefix, optarg);
 #			endif
@@ -498,9 +500,7 @@ int options_parse(options_s *options, device_s *dev, encoder_s *enc, stream_s *s
 		}
 	ADD_SINK("JPEG", sink);
 	ADD_SINK("RAW", raw_sink);
-#	ifdef WITH_OMX
 	ADD_SINK("H264", h264_sink);
-#	endif
 #	undef ADD_SINK
 
 #	ifdef WITH_SETPROCTITLE
@@ -539,18 +539,18 @@ static int _parse_resolution(const char *str, unsigned *width, unsigned *height,
 }
 
 static void _features(void) {
-#	ifdef WITH_OMX
-	puts("+ WITH_OMX");
-#	else
-	puts("- WITH_OMX");
-#	endif
-
 #	ifdef WITH_GPIO
 	puts("+ WITH_GPIO");
 #	else
 	puts("- WITH_GPIO");
 #	endif
 
+#	ifdef WITH_SYSTEMD
+	puts("+ WITH_SYSTEMD");
+#	else
+	puts("- WITH_SYSTEMD");
+#	endif
+
 #	ifdef WITH_PTHREAD_NP
 	puts("+ WITH_PTHREAD_NP");
 #	else
@@ -572,10 +572,10 @@ static void _features(void) {
 
 static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, server_s *server) {
 #	define SAY(_msg, ...) fprintf(fp, _msg "\n", ##__VA_ARGS__)
-	SAY("\nuStreamer - Lightweight and fast MJPG-HTTP streamer");
+	SAY("\nuStreamer - Lightweight and fast MJPEG-HTTP streamer");
 	SAY("═══════════════════════════════════════════════════");
 	SAY("Version: %s; license: GPLv3", VERSION);
-	SAY("Copyright (C) 2018-2021 Maxim Devaev <mdevaev@gmail.com>\n");
+	SAY("Copyright (C) 2018-2022 Maxim Devaev <mdevaev@gmail.com>\n");
 	SAY("Capturing options:");
 	SAY("══════════════════");
 	SAY("    -d|--device </dev/path>  ───────────── Path to V4L2 device. Default: %s.\n", dev->path);
@@ -592,7 +592,7 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 	SAY("    -z|--min-frame-size <N>  ───────────── Drop frames smaller then this limit. Useful if the device");
 	SAY("                                           produces small-sized garbage frames. Default: %zu bytes.\n", dev->min_frame_size);
 	SAY("    -n|--persistent  ───────────────────── Don't re-initialize device on timeout. Default: disabled.\n");
-	SAY("    -t|--dv-timings  ───────────────────── Enable DV timings querying and events processing");
+	SAY("    -t|--dv-timings  ───────────────────── Enable DV-timings querying and events processing");
 	SAY("                                           to automatic resolution change. Default: disabled.\n");
 	SAY("    -b|--buffers <N>  ──────────────────── The number of buffers to receive data from the device.");
 	SAY("                                           Each buffer may processed using an independent thread.");
@@ -603,19 +603,16 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 	SAY("                                           Note: If HW encoding is used (JPEG source format selected),");
 	SAY("                                           this parameter attempts to configure the camera");
 	SAY("                                           or capture device hardware's internal encoder.");
-	SAY("                                           It does not re-encode MJPG to MJPG to change the quality level");
-	SAY("                                           for sources that already output MJPG.\n");
+	SAY("                                           It does not re-encode MJPEG to MJPEG to change the quality level");
+	SAY("                                           for sources that already output MJPEG.\n");
 	SAY("    -c|--encoder <type>  ───────────────── Use specified encoder. It may affect the number of workers.");
 	SAY("                                           Available:");
-	SAY("                                             * CPU  ── Software MJPG encoding (default);");
-#	ifdef WITH_OMX
-	SAY("                                             * OMX  ── GPU hardware accelerated MJPG encoding with OpenMax;");
-#	endif
-	SAY("                                             * HW  ─── Use pre-encoded MJPG frames directly from camera hardware.");
-	SAY("                                             * NOOP  ─ Don't compress MJPG stream (do nothing).\n");
-#	ifdef WITH_OMX
+	SAY("                                             * CPU  ──────── Software MJPEG encoding (default);");
+	SAY("                                             * HW  ───────── Use pre-encoded MJPEG frames directly from camera hardware;");
+	SAY("                                             * M2M-VIDEO  ── GPU-accelerated MJPEG encoding using V4L2 M2M video interface;");
+	SAY("                                             * M2M-IMAGE  ── GPU-accelerated JPEG encoding using V4L2 M2M image interface;");
+	SAY("                                             * NOOP  ─────── Don't compress MJPEG stream (do nothing).\n");
 	SAY("    -g|--glitched-resolutions <WxH,...>  ─ It doesn't do anything. Still here for compatibility.\n");
-#	endif
 	SAY("    -k|--blank <path>  ─────────────────── Path to JPEG file that will be shown when the device is disconnected");
 	SAY("                                           during the streaming. Default: black screen 640x480 with 'NO SIGNAL'.\n");
 	SAY("    -K|--last-as-blank <sec>  ──────────── Show the last frame received from the camera after it was disconnected,");
@@ -628,6 +625,7 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 	SAY("    --device-timeout <sec>  ────────────── Timeout for device querying. Default: %u.\n", dev->timeout);
 	SAY("    --device-error-delay <sec>  ────────── Delay before trying to connect to the device again");
 	SAY("                                           after an error (timeout for example). Default: %u.\n", stream->error_delay);
+	SAY("    --m2m-device </dev/path>  ──────────── Path to V4L2 M2M encoder device. Default: auto select.\n");
 	SAY("Image control options:");
 	SAY("══════════════════════");
 	SAY("    --image-default  ────────────────────── Reset all image settings below to default. Default: no change.\n");
@@ -652,6 +650,9 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 	SAY("    -U|--unix <path>  ─────────── Bind to UNIX domain socket. Default: disabled.\n");
 	SAY("    -D|--unix-rm  ─────────────── Try to remove old UNIX socket file before binding. Default: disabled.\n");
 	SAY("    -M|--unix-mode <mode>  ────── Set UNIX socket file permissions (like 777). Default: disabled.\n");
+#	ifdef WITH_SYSTEMD
+	SAY("    -S|--systemd  ─────────────── Bind to systemd socket for socket activation.\n");
+#	endif
 	SAY("    --user <name>  ────────────── HTTP basic auth user. Default: disabled.\n");
 	SAY("    --passwd <str>  ───────────── HTTP basic auth passwd. Default: empty.\n");
 	SAY("    --static <path> ───────────── Path to dir with static files instead of embedded root index page.");
@@ -661,25 +662,24 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 	SAY("                                  the CPU loading. Don't use this option with analog signal sources");
 	SAY("                                  or webcams, it's useless. Default: disabled.\n");
 	SAY("    -R|--fake-resolution <WxH>  ─ Override image resolution for the /state. Default: disabled.\n");
-	SAY("    --tcp-nodelay  ────────────── Set TCP_NODELAY flag to the client /stream socket. Ignored for --unix.");
+	SAY("    --tcp-nodelay  ────────────── Set TCP_NODELAY flag to the client /stream socket. Only for TCP socket.");
 	SAY("                                  Default: disabled.\n");
 	SAY("    --allow-origin <str>  ─────── Set Access-Control-Allow-Origin header. Default: disabled.\n");
 	SAY("    --server-timeout <sec>  ───── Timeout for client connections. Default: %u.\n", server->timeout);
 #	define ADD_SINK(_name, _opt) \
 		SAY(_name " sink options:"); \
 		SAY("══════════════════"); \
-		SAY("    --" _opt "sink <name>  ─────────── Use the shared memory to sink " _name " frames. Default: disabled.\n"); \
-		SAY("    --" _opt "sink-mode <mode>  ────── Set " _name " sink permissions (like 777). Default: 660.\n"); \
-		SAY("    --" _opt "sink-rm  ─────────────── Remove shared memory on stop. Default: disabled.\n"); \
-		SAY("    --" _opt "sink-client-ttl <sec>  ─ Client TTL. Default: 10.\n"); \
-		SAY("    --" _opt "sink-timeout <sec>  ──── Timeout for lock. Default: 1.\n");
+		SAY("    --" _opt "sink <name>  ──────────── Use the shared memory to sink " _name " frames. Default: disabled.\n"); \
+		SAY("    --" _opt "sink-mode <mode>  ─────── Set " _name " sink permissions (like 777). Default: 660.\n"); \
+		SAY("    --" _opt "sink-rm  ──────────────── Remove shared memory on stop. Default: disabled.\n"); \
+		SAY("    --" _opt "sink-client-ttl <sec>  ── Client TTL. Default: 10.\n"); \
+		SAY("    --" _opt "sink-timeout <sec>  ───── Timeout for lock. Default: 1.\n");
 	ADD_SINK("JPEG", "")
 	ADD_SINK("RAW", "raw-")
-#	ifdef WITH_OMX
 	ADD_SINK("H264", "h264-")
-	SAY("    --h264-bitrate <kbps>  ──────── H264 bitrate in Kbps. Default: %u.\n", stream->h264_bitrate);
-	SAY("    --h264-gop <N>  ─────────────── Intarval between keyframes. Default: %u.\n", stream->h264_gop);
-#	endif
+	SAY("    --h264-bitrate <kbps>  ───────── H264 bitrate in Kbps. Default: %u.\n", stream->h264_bitrate);
+	SAY("    --h264-gop <N>  ──────────────── Intarval between keyframes. Default: %u.\n", stream->h264_gop);
+	SAY("    --h264-m2m-device </dev/path>  ─ Path to V4L2 M2M encoder device. Default: auto select.\n");
 #	undef ADD_SINK
 #	ifdef WITH_GPIO
 	SAY("GPIO options:");
@@ -697,6 +697,8 @@ static void _help(FILE *fp, device_s *dev, encoder_s *enc, stream_s *stream, ser
 #	ifdef HAS_PDEATHSIG
 	SAY("    --exit-on-parent-death  ─────── Exit the program if the parent process is dead. Default: disabled.\n");
 #	endif
+	SAY("    --exit-on-no-clients <sec> ──── Exit the program if there have been no stream or sink clients");
+	SAY("                                    or any HTTP requests in the last N seconds. Default: 0 (disabled)\n");
 #	ifdef WITH_SETPROCTITLE
 	SAY("    --process-name-prefix <str>  ── Set process name prefix which will be displayed in the process list");
 	SAY("                                    like '<str>: ustreamer --blah-blah-blah'. Default: disabled.\n");
